//
// Copyright 2002 by Craig Stuart Sapp, All Rights Reserved.
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Feb 11 19:48:50 PST 2002
// Last Modified: Tue Feb 26 17:29:02 PST 2002
// Last Modified: Tue Mar 26 09:56:28 PST 2002 (added enum)
// Filename:      ...sig/src/sigInfo/ScoreRecord.h
// Web Address:   http://sig.sapp.org/include/sigInfo/ScoreRecord.h
// Syntax:        C++ 
//
// Description:   A SCORE item parameter class
//

#ifndef _SCORERECORD_H_INCLUDED
#define _SCORERECORD_H_INCLUDED

#include "ParameterItem.h"

#ifndef OLDCPP
   #include <iostream>
   using namespace std;
#else
   #include <iostream.h>
#endif

typedef enum {
	S_Note = 1,
	S_Rest,
	S_Clef,
	S_Line,
	S_Slur,
	S_Beam,
	S_Trill,
	S_Staff,
	S_Symbol,
	S_Number,
	S_User,
	S_Special,
	S_BadLuck,
	S_Barline,
	S_ImportedEPSGraphic,
	S_Text,
	S_KeySignature,
	S_MeterSignature
} PARM1VALUE;


class ScoreRecord : public ParameterItem {
   public:
                     ScoreRecord       (void);
                    ~ScoreRecord       ();

      void           printAscii        (ostream& out, int verboseQ = 0);
      void           clear             (void);
      int            writeBinary       (ostream& out);
      void           readBinary        (istream& instream, int pcount);
      void           shrink            (void);
      const char*    getTextData       (void);
      void           setTextData       (const char* buffer);

      // booleans for identifying code items.
      int            isNoteItem        (void);
      int            isRestItem        (void);
      int               hasDurationQ   (void);
      int            isClefItem        (void);
      int            isLineItem        (void);
      int            isSlurItem        (void);
      int            isBeamItem        (void);
      int            isTrillItem       (void);
      int            isStaffItem       (void);
      int            isSymbolItem      (void);
      int            isNumberItem      (void);
      int            isUserItem        (void);
      int            isShapeItem       (void);
      int            isBarlineItem     (void);
      int            isPostscriptItem  (void);
      int            isTextItem        (void);
      int            isKeysigItem      (void);
      int            isTimesigItem     (void);
   
      int            getPitch          (void);
      void           setPitch          (int aPitch);
      float          getStartOffset    (void);
      void           setStartOffset    (float aDuration);
      int            getVoice          (void);
      void           setVoice          (int aVoice);

   private:
      void           writeLittleEndian (ostream& out, float number);
      float          readLittleEndian  (istream& instream);
      void           adjustBuffer      (char *buffer);

      char*          textdata;         // for storing SCORE text data
      Array<float>   analysisData;     // analysis data: duration and pitch
                                       // P1 = pitch (-1 for rest)
                                       // P2 = duration offset from the start 
                                       //      of the staff
                                       // P3 = voice number 
};


#endif /* _SCORERECORD_H_INCLUDED */


